#include <JGL/types/Light2D.h>
#include <J3ML/LinearAlgebra/Vector2.hpp>
#include <algorithm>

JGL::J2D::PointLight::PointLight(const Vector2& position, const Color4& ambient, const Color4& diffuse, float constant_attenuation, float linear_attenuation, float quadratic_attenuation) {
    light_pos = Vector4(position.x, position.y, 1, 1);
    this->ambient = ambient;
    this->diffuse = diffuse;
    this->constant_attenuation = constant_attenuation;
    this->linear_attenuation = linear_attenuation;
    this->quadratic_attenuation = quadratic_attenuation;
}

float JGL::J2D::PointLight::MeasureIntensityAtPosition(const Vector2& position) const {
    float d = Vector2::Distance( Vector2(light_pos.x, light_pos.y), position);

    // Fix possible divide by zero if the position we're measuring happens to be directly on-top of the light.
    if (d == 0)
        d = 0.00001;

    return 1.f / (constant_attenuation + linear_attenuation * d + quadratic_attenuation * d * d);
}

Vector2 JGL::J2D::Light::Position() const {
    return { light_pos.x, light_pos.y };
}

Color4 JGL::J2D::Light::Ambient() const {
    return ambient;
}

Color4 JGL::J2D::Light::Diffuse() const {
    return diffuse;
}

float JGL::J2D::Light::Constant() const {
    return constant_attenuation;
}

float JGL::J2D::Light::Linear() const {
    return linear_attenuation;
}

float JGL::J2D::Light::Quadratic() const {
    return quadratic_attenuation;
}

void JGL::J2D::Light::Position(const Vector2& new_position) {
    light_pos.x = new_position.x;
    light_pos.y = new_position.y;
}

void JGL::J2D::Light::Ambient(const Color4& new_ambient) {
    ambient = new_ambient;
}

void JGL::J2D::Light::Diffuse(const Color4& new_diffuse) {
    diffuse = new_diffuse;
}

void JGL::J2D::Light::Constant(float new_constant_attenuation) {
    constant_attenuation = std::clamp(new_constant_attenuation, 0.0f, 1.0f);
}

void JGL::J2D::Light::Linear(float new_linear_attenuation) {
    linear_attenuation = std::clamp(new_linear_attenuation, 0.0f, 1.0f);
}

void JGL::J2D::Light::Quadratic(float new_quadratic_attenuation) {
    quadratic_attenuation = std::clamp(new_quadratic_attenuation, 0.0f, 1.0f);
}
